--[[****************************版权所有,请勿修改!****************************]]
--插件作者: 紫猫学院
--联系QQ: 345911220
--主打功能: 文本文件操作
--使用方法: 在按键精灵中采用 Import "zmFile.lua" 导入插件后，再用 zmFile.函数名 即可调用
--[[****************************版权所有,请勿修改!****************************]]


--[[**************************************************************************]]
--[[****************************** 全局数据 **********************************]]
--[[**************************************************************************]]

local _ZM_ = {}
local zimao = {}
QMPlugin = zimao 

_ZM_._VERSION_       = "3.1930" -- 对外版本号


--[[**************************************************************************]]
--[[****************************** 基础函数 **********************************]]
--[[**************************************************************************]]

local function traceprint(...)
    if QMPlugin then
        print(...)
    else
        local line = select(1, ...)
        if type(line) == "string" and line:match("^%_%d+$") then
            LuaAuxLib.TracePrint(line .. ":", table.concat({...}, " ", 2, select("#", ...)))
        elseif line == ":" and #{...} > 1 then
            LuaAuxLib.TracePrint(":", table.concat({...}, " ", 2, select("#", ...)))
        else
            LuaAuxLib.TracePrint(":","紫猫学院文件插件：", ...)
        end
    end
end


local function try(block)
    local tablejoin = function (...)
        local result = {}
        for _, t in ipairs({...}) do
            if type(t) == "table" then
                for k, v in pairs(t) do
                    if type(k) == "number" then table.insert(result, v)
                    else result[k] = v end
                end
            else
                table.insert(result, t)
            end
        end
        return result
    end
        
    -- get the try function
    local try = block[1]
    assert(try)
    
    -- get catch and finally functions
    local funcs = tablejoin(block[2] or {}, block[3] or {})
    
    -- try to call it
    local result_error = {}
    local results = {pcall(try)}
    if not results[1] then
        -- run the catch function
        if funcs and funcs.catch then
            result_error = {funcs.catch(results[2])}
        end
    end

    -- run the finally function
    if funcs and funcs.finally then
        local result_fin = {funcs.finally(table.unpack(results))}
        if #result_fin > 0 then
            return table.unpack(result_fin)
        end
    end

    -- ok?
    if results[1] and #results > 1 then
        return table.unpack(results, 2, #results)
    else
        if #result_error > 0 then
            return table.unpack(result_error)
        else
            return nil
        end
    end
end
local function catch(block)
    -- get the catch block function
    return {catch = block[1]}
end
local function finally(block)
    -- get the finally block function
    return {finally = block[1]}
end

function _ZM_.TableClear(t)
    return try{
        function ()
            for k in next, t do
                rawset(t, k, nil) 
            end
            return t
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：TableClear()，错误信息：", errors)
            end
        }
    }
end

function _ZM_.ConvBase(number, frombase, tobase)
    return try {
        function ()
            local digits, num, ptr, n, c
            num = 0; ptr = ""
            digits = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
            
            assert(62 >= frombase and frombase >= 2, "起始进制超过2至62进制范围。")
            assert(62 >= tobase and tobase >= 2 , "目标进制超过2至62进制范围。")
            
            number = tostring(number)
            assert(type(number)=="string", "待转换内容参数不是字符串型")
            n = string.len(number)

            for i = 1, n do
                c = string.sub(number, i, i)
                if (c >= "0") and (c <= "9") then
                    c = c - "0"
                elseif (c >= "A") and (c <= "Z") then
                        if frombase<=36 and tobase<=36 then
                        c = string.byte(c) - string.byte("A") + 10
                    else
                        c = string.byte(c) - string.byte("A") + 36
                    end
                elseif (c >= "a") and (c <= "z") then
                    c = string.byte(c) - string.byte("a") + 10
                else
                    c = frombase
                end

                if c < frombase then
                    num = num * frombase + c
                end
            end

            repeat
                ptr = ptr .. string.sub(digits, num % tobase + 1, num % tobase + 1)
                num = math.floor(num / tobase)
            until num == 0

            ptr = string.reverse(ptr)
            return ptr
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：ConvBase()，错误信息：", errors)
            end
        }
    }
end

function _ZM_.ConvEncoding(text, from, to)
    return try {
        function()
            local iconv = require("iconv")
            if from == "电脑" then from = "gb18030" end
            if to == "电脑" then to = "gb18030" end
            if from == "手机" then from = "utf-8" end
            if to == "手机" then to = "utf-8" end
            
            local cd = iconv.new(to, from)
            local ostr, err = cd:iconv(text)
            if err == iconv.ERROR_INCOMPLETE then
                error("参数不完整")
            elseif err == iconv.ERROR_INVALID then
                error("参数无效")
            elseif err == iconv.ERROR_NO_MEMORY then
                error("内存无法分配")
            elseif err == iconv.ERROR_UNKNOWN then
                error("未知错误")
            end
            return ostr
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：ConvEncoding()，错误信息：", errors)
            end
        }
    }
end

function _ZM_.clearbom(str)
    -- str = str:gsub("^" .. string.char(0xEF, 0xBB, 0xBF), "")
    if str:sub(1, 3) == string.char(0xEF, 0xBB, 0xBF) then
        str = str:sub(4, -1)
    end
    return str
end

--[[**************************************************************************]]
--[[****************************** 文件操作 **********************************]]
--[[**************************************************************************]]

--[=[
@fname:Create
@cname:创建文件
@format:[文件路径]
@tname:创建 $1 文件
@note:创建一个空内容的文件, 若文件已存在则清空该文件内容
@ret:文件路径, string, 返回文件路径, 失败返回null
@arg:文件路径, string, 可选, 指定文件路径, 省略默认为随机文件名
@exp:TracePrint "创建文件:", zmFile.Create()
--]=]
function zimao.Create(filename)
    return try {
        function()
            local FileTempPath
            if filename then
                FileTempPath = filename
            else
                FileTempPath = __MQM_RUNNER_LOCAL_PATH_GLOBAL_NAME__ .. os.tmpname():match("[^/]+$")
            end
            zimao.Write(FileTempPath, "")
            return FileTempPath
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.Create()，错误信息：", errors)
            end
        }
    }
end


_ZM_.gFileAttr = {}
_ZM_.gFileAttr.default = {encoding="utf-8", fileencoding={}}
_ZM_.gFileAttr.mt = {__index = _ZM_.gFileAttr.default}
--[=[
@fname:SetEncoding
@cname:设置读写编码
@format:文件编码[, 文件路径]
@tname:设置 $1 为 $2 的读写编码
@note:设置文件的读写编码, 可以指定单独某个文件特殊编码
@ret:设置后的编码, string, 返回设置后的编码, 失败返回null
@arg:文件编码, string, 要设置的编码类型, 电脑上编辑的文件可以设置为"电脑"或"gb18030", 手机上编辑的文件可以设置为"手机"或"utf-8", 需要自动判断则设置为"自动"或"auto"
@arg:文件路径, string, 可选, 要设置编码的文件路径, 省略不填为null, 表示所有文件
@exp:TracePrint "设置编码:", zmFile.SetEncoding("电脑")
--]=]
function zimao.SetEncoding(encoding, path)
    return try {
        function()
            if path then
                _ZM_.gFileAttr.default.fileencoding[path] = encoding
            else
                _ZM_.gFileAttr.default.encoding = encoding
            end
            return encoding
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.SetEncoding()，错误信息：", errors)
            end
        }
    }
end
--[=[
@fname:GetEncoding
@cname:获取读写编码
@format:[文件路径]
@tname:获取 $1 的读写编码
@note:获取插件的读写编码, 填写路径后可以获取该文件的编码
@ret:文件编码, string, 返回文件编码, 失败返回null
@arg:文件路径, string, 可选, 要获取编码的文件路径, 注意, 当文件内容很多时, 获取速度可能会比较慢, 省略不填为null, 表示插件默认读写编码
@exp:TracePrint "获取全局编码:", zmFile.GetEncoding()
--]=]
function zimao.GetEncoding(path)
    return try {
        function()
            if path then
                return _ZM_.AutoEncoding(path)
            else
                return _ZM_.gFileAttr.default.encoding
            end
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.GetEncoding()，错误信息：", errors)
            end
        }
    }
end

function _ZM_.AutoEncoding(path)
    local f
    return try {
        function ()
            local encoding = "utf-8"
            local isFirst = true
            local args = setmetatable({path=path}, _ZM_.gFileAttr.mt)
            
            if not(args.fileencoding[args.path]) or args.fileencoding[args.path] == "自动" or args.fileencoding[args.path]:lower() == "auto" then
                traceprint("正在判断文件编码中...该过程可能需要较长时间...")
                f = io.open(args.path, "r")
                if f ~= nil then
                    for text in f:lines() do
                        if isFirst and text:match("^" .. string.char(0xEF, 0xBB, 0xBF)) then
                            encoding = "utf-8"
                            break
                        else
                            for i=1,#text do
                                local t = {}
                                t[1] = _ZM_.ConvBase(text:byte(i,i), 10, 2)
                                t[2] = _ZM_.ConvBase(text:byte(i+1,i+1), 10, 2)
                                t[3] = _ZM_.ConvBase(text:byte(i+2,i+2), 10, 2)
                                if t[1]:match("^1110%d%d%d%d") then
                                    if not(t[2]:match("^10%d%d%d%d%d%d") and t[3]:match("^10%d%d%d%d%d%d")) then
                                        encoding = "gb18030"
                                        break
                                    end
                                elseif t[1]:match("^110%d%d%d%d%d") then
                                    if t[2]:match("^10%d%d%d%d%d%d") then
                                        encoding = "gb18030"
                                        break
                                    end
                                end
                            end
                        end
                        if encoding == "gb18030" then
                            break
                        end
                        isFirst = false
                    end
                    
                    f:close()
                else
                    encoding = "utf-8"
                end
                
                _ZM_.gFileAttr.default.fileencoding[path] = encoding
            else
                encoding = args.fileencoding[args.path]
            end
            
            return encoding
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：AutoEncoding()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end


--[=[
@fname:Read
@cname:读取文本文件
@format:文件路径[, 文件编码]
@tname:以 $2 编码读取 $1 文件
@note:读取文件所有内容, 支持设置文件编码
@ret:文本内容, string, 返回该文件的所有文本内容, 失败返回null
@arg:文件路径, string, 要操作的文件路径
@arg:文件编码, string, 可选, 填写"电脑"或"gb18030"对应电脑文件, 填写"手机"或"utf-8"对应手机文件, 填写"自动"或"auto"将自动判断编码, 省略默认为zmFile.SetEncoding()设置的编码
@exp:TracePrint zmFile.Read("/sdcard/电脑文件.txt", "电脑") //可以直接读取电脑传过来的文件
--]=]
function zimao.Read(path, encoding)
    local f
    return try {
        function()
            local args = setmetatable({path=path, encoding=encoding}, _ZM_.gFileAttr.mt)
            
            if args.encoding == "auto" or args.encoding == "自动" then
                args.encoding = _ZM_.AutoEncoding(args.path)
            end
            
            
            f = io.open(args.path, "r")
            if f == nil then
                return nil
            end
            local ret = f:read("*all")
            f:close()
            
            if args.encoding ~= "utf-8" then
                ret = _ZM_.ConvEncoding(ret, args.encoding, "utf-8")
            end
            
            if args.encoding == "utf-8" then ret = _ZM_.clearbom(ret) end
            return ret
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.Read()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end

--[=[
@fname:ReadLine
@cname:读取指定行文本
@format:文件路径, 指定行[, 文件编码]
@tname:以 $3 编码读取 $1 文件的第 $2 行内容
@note:读取指定行文本内容, 支持倒数行数, 超出行数返回null, 支持设置文件编码
@ret:文本内容, string, 返回指定行的文本内容, 失败返回null
@arg:文件路径, string, 要操作的文件路径
@arg:指定行, number, 表示读取第几行内容, 负数为倒数第几行内容
@arg:文件编码, string, 可选, 填写"电脑"或"gb18030"对应电脑文件, 填写"手机"或"utf-8"对应手机文件, 填写"自动"或"auto"将自动判断编码, 省略默认为zmFile.SetEncoding()设置的编码
@exp:TracePrint zmFile.ReadLine("/sdcard/紫猫学院.txt", 2) //读取第2行内容
@exp:TracePrint zmFile.ReadLine("/sdcard/紫猫学院.txt", -5) //读取倒数第5行内容
--]=]
function zimao.ReadLine(path, line, encoding)
    local args, f = {path=path, line=line, encoding=encoding}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            if args.encoding == "auto" or args.encoding == "自动" then
                args.encoding = _ZM_.AutoEncoding(args.path)
            end
            
            f = io.open(args.path, "r")
            if f == nil then
                return nil
            end
            
            if args.line < 0 then
                local l = 0
                for _ in f:lines() do
                    l = l + 1
                end
                f:seek("set")
                args.line = l + args.line + 1
            end
            
            local ll, text = 0
            for c in f:lines() do
                ll = ll + 1
                if ll == args.line then
                    if args.encoding ~= "utf-8" then
                        c = _ZM_.ConvEncoding(c, args.encoding, "utf-8")
                    end
                    text = c:gsub("[\r\n]", "")
                    break
                end
            end
            
            f:close()
            if args.encoding == "utf-8" then text = _ZM_.clearbom(text) end
            return text
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.ReadLine()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end

--[=[
@fname:ReadLines
@cname:读取文本数组
@format:文件路径[, 文件编码]
@tname:以 $2 编码读取 $1 文件内容保存到数组
@note:读取文件所有内容保存到一维数组中, 支持设置文件编码
@ret:文本数组, table, 返回文件每一行内容保存到一维数组中
@arg:文件路径, string, 要操作的文件路径
@arg:文件编码, string, 可选, 填写"电脑"或"gb18030"对应电脑文件, 填写"手机"或"utf-8"对应手机文件, 填写"自动"或"auto"将自动判断编码, 省略默认为zmFile.SetEncoding()设置的编码
@exp:Dim txtArr = zmFile.ReadLines("/sdcard/紫猫学院.txt") //读取所有内容保存到数组txtArr中
@exp:Dim txtArr = zmFile.ReadLines("/sdcard/紫猫学院.txt", "电脑") //以电脑文件编码读取所有内容保存到数组txtArr中
--]=]
function zimao.ReadLines(path, encoding)
    local args, f = {path=path, encoding=encoding}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            if args.encoding == "auto" or args.encoding == "自动" then
                args.encoding = _ZM_.AutoEncoding(args.path)
            end
            
            f = io.open(args.path, "r")
            if f == nil then
                return nil
            end
            local t = {}
            for c in f:lines() do
                if args.encoding ~= "utf-8" then
                    c = _ZM_.ConvEncoding(c, args.encoding, "utf-8")
                end
                c = c:gsub("[\r\n]", "")
                if args.encoding == "utf-8" then c = _ZM_.clearbom(c) end
                t[#t+1] = c
            end
            f:close()
            return t
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.ReadLines()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end


function _ZM_.Split(str, delim, plain)
    return try {
        function()
            str = type(str) == "number" and tostring(str) or str
            assert(type(str) == "string", "第1个参数不是字符串型")
            if type(delim) == "string" then
                local pos, result, s, e, l = 1, {}, 0, 0, str:len()
                plain = not(plain)
                if str:find(delim, pos, plain) == nil or delim == "" then
                    return { str }
                end
                
                while true do
                    s, e = str:find(delim, pos, plain)
                    if s then
                        local ss = str:sub(pos, s-1)
                        table.insert(result, ss)
                        pos = e + 1
                    else
                        local ss = str:sub(pos, l)
                        table.insert(result, ss)
                        break
                    end
                end
                return result
            else
                local t, l = {}
                if type(delim) == "number" and delim > 0 then
                    l = delim
                else
                    l = 1
                end
                for i=1,LuaAuxLib.UTF8_Length(str),l do
                    table.insert(t, (LuaAuxLib.UTF8_Mid(str, i, l)))
                end
                return t
            end
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：Split()，错误信息：", errors)
            end
        }
    }
end

--[=[
@fname:ReadForm
@cname:读取文本表格
@format:文件路径, 分隔符[, 文件编码]
@tname:以 $3 编码读取 $1 文件内容按 $2 分隔保存到二维数组
@note:按指定分隔符读取文件所有内容保存到二维数组中, 支持设置文件编码
@ret:文本数组, table, 返回每一行被分隔后的二维数组文本
@arg:文件路径, string, 要操作的文件路径
@arg:分隔符, string或number, 字符串表示每行文本中的分隔符, 数值型表示按长度分隔
@arg:文件编码, string, 可选, 填写"电脑"或"gb18030"对应电脑文件, 填写"手机"或"utf-8"对应手机文件, 填写"自动"或"auto"将自动判断编码, 省略默认为zmFile.SetEncoding()设置的编码
@exp:Dim path = "/sdcard/紫猫学院.txt"$n zmFile.Write path, {"帐号1====密码1====大区1", "帐号2====密码2====大区2", "帐号3====密码3====大区3"}$n Dim txts = zmFile.ReadForm(path, "====")$n TracePrint zmFile.VarInfo(txts)
--]=]
function zimao.ReadForm(path, sep, encoding)
    local args, f = {path=path, sep=sep, encoding=encoding}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            
            if args.encoding == "auto" or args.encoding == "自动" then
                args.encoding = _ZM_.AutoEncoding(args.path)
            end
            
            f = io.open(args.path, "r")
            if f == nil then
                return nil
            end
            local t = {}
            for c in f:lines() do
                if args.encoding ~= "utf-8" then
                    c = _ZM_.ConvEncoding(c, args.encoding, "utf-8")
                end
                if args.encoding == "utf-8" then c = _ZM_.clearbom(c) end
                c = c:gsub("[\r\n]", "")
                local tt = _ZM_.Split(c, args.sep)
                t[#t+1] = tt
            end
            f:close()
            return t
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.ReadForm()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end

--[=[
@fname:WriteForm
@cname:以表格方式写入
@format:文件路径, 二维数组, 分隔符
@tname:以分隔符 $3 覆盖写入 $2 到文件 $1 中
@note:覆盖写入二维数组到文件中, 文件编码为zmFile.SetEncoding()设置的编码
@ret:文本, string, 返回写入的文本, 失败返回null
@arg:文件路径, string, 要操作的文件路径
@arg:二维数组, table, 与zmFile.ReadForm()相反, 写入读取到的二维数组到文件中
@arg:分隔符, string, 同行文本的分隔符内容
@exp:zmFile.WriteForm("/sdcard/紫猫学院.txt", {{"帐号1", "密码1"}, {"帐号2", "密码2"}, {"帐号3", "密码3"}}, "====")
--]=]
function zimao.WriteForm(path, form, sep)
    local args, f = {path=path, form=form, sep=sep}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            
            if args.encoding == "auto" or args.encoding == "自动" then
                args.encoding = _ZM_.AutoEncoding(args.path)
            end
            
            f = io.open(args.path, "w")
            if f == nil then
                return nil
            end
            
            args.text = {}
            
            for _, v in ipairs(args.form) do
                local text = table.concat(v, args.sep)
                
                if args.encoding ~= "utf-8" then
                    text = _ZM_.ConvEncoding(text, "utf-8", args.encoding)
                end
                args.text[#args.text+1] = text
            end
            
            if args.encoding ~= "utf-8" then
                args.newline = "\r\n"
            else
                args.newline = "\n"
            end
            args.text = table.concat(args.text, args.newline)
            f:write(args.text)
            f:close()
            return args.text
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.WriteForm()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end

function _ZM_.ArrayFromString(str)
    local bytes = {};

	local i=1;
	local byte = string.byte(str,i);
	while #str >= i do
		bytes[i] = byte;
		i = i + 1;
		byte = string.byte(str,i);
	end

	return bytes;
end

--[=[
@fname:ReadBinary
@cname:读取二进制文件
@format:文件路径[, 读取长度][, 返回十六进制数组]
@tname:读取 $1 文件二进制内容
@note:以二进制读取文件内容
@ret:二进制, string, 返回读取到的二进制内容
@arg:文件路径, string, 要操作的文件路径
@arg:读取长度, number, 可选, 要读取的字节长度, 省略默认为全部读取
@arg:返回十六进制数组, table, 可选, 数组变量, 用于返回十六进制数组结果
@exp:Dim bin = zmFile.ReadBinary("/sdcard/截图1.png") //读取二进制数据, 一般无法直接输出, 可以用Len查看长度
--]=]
function zimao.ReadBinary(path, length, hex)
    local args, f = {path=path,length=length, hex=hex}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            
            f = io.open(args.path, "rb")
            if f == nil then
                return nil
            end
            
            if type(args.length) == "table" then
                args.length, args.hex = args.hex, args.length
            end
            
            if args.length == nil then
                args.length = "*a"
            end
            
            local ret = f:read(args.length)
            f:close()
            
            if type(args.hex) == "table" then
                _ZM_.TableClear(args.hex)
                for k, v in pairs(_ZM_.ArrayFromString(ret)) do
                    args.hex[k] = v
                end
            end
            
            return ret
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.ReadBinary()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end


--[=[
@fname:Write
@cname:覆盖写入文本
@format:文件路径, 内容[, ...]
@tname:覆盖写入 $2 到文件 $1 中
@note:覆盖写入文本内容到文件中, 文件编码为zmFile.SetEncoding()设置的编码
@ret:文本数组, table, 返回写入的文本数组, 失败返回null
@arg:文件路径, string, 要操作的文件路径
@arg:内容, string或table, 支持一个数组参数或者多个字符串参数$n 如果内容是一个数组参数, 则表示换行写入每个元素$n 如果内容是多个字符串参数, 则表示不换行依次写入每个字符串
@exp:zmFile.Write("/sdcard/紫猫学院.txt", "紫,", "猫", "学", "院") //不换行覆盖写入
@exp:zmFile.Write("/sdcard/紫猫学院.txt", {"紫,", "猫", "学", "院"}) //换行覆盖写入
--]=]
function zimao.Write(path, ...)
    local args, f = {path=path, texts={...}}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            
            if args.encoding == "auto" or args.encoding == "自动" then
                args.encoding = _ZM_.AutoEncoding(args.path)
            end
            
            f = io.open(args.path, "w")
            if f == nil then
                return nil
            end
            
            if type(args.texts[1]) == "table" then
                if args.encoding ~= "utf-8" then
                    args.sep = "\r\n"
                else
                    args.sep = "\n"
                end
                args.texts = args.texts[1]
            end
            
            args.text = table.concat(args.texts, args.sep)
            
            if args.encoding ~= "utf-8" then
                args.text = _ZM_.ConvEncoding(args.text, "utf-8", args.encoding)
            end
            
            f:write(args.text)
            f:close()
            return args.texts
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.Write()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end

--[=[
@fname:WriteLine
@cname:插入指定行文本
@format:文件路径, 指定行, 内容[, ...]
@tname:追加 $3 到文件 $1 的第 $2 行中
@note:追加写入到指定行, 文件编码为zmFile.SetEncoding()设置的编码
@ret:文本数组, table, 返回写入后的文本数组, 失败返回null
@arg:文件路径, string, 要操作的文件路径
@arg:指定行, number, 表示插入第几行内容, 负数为倒数第几行, 0为末尾新增一行, 不能超出最大行
@arg:内容, string或table, 支持一个数组参数或者多个字符串参数$n 如果内容是一个数组参数, 则表示换行写入每个元素$n 如果内容是多个字符串参数, 则表示不换行依次写入每个字符串
@exp:zmFile.WriteLine("/sdcard/紫猫学院.txt", 2, "好好学习, 天天向上") //在文件第2行插入文本
--]=]
function zimao.WriteLine(path, line, ...)
    local args, f = {path=path, line=line, texts={...}}
	return try {        
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            
            if args.encoding == "auto" or args.encoding == "自动" then
                args.encoding = _ZM_.AutoEncoding(args.path)
            end
            
            f = io.open(args.path, "r+")
            if f == nil then
                error("文件不存在!")
            end
            
            -- 统计倒数行数
            local l = 0
            if args.line <= 0 then
                for _ in f:lines() do
                    l = l + 1
                end
                f:seek("set")
                args.line = l + args.line + 1
            end
            
            local ll, t, ischange = 0, {}
            --写入函数
            local w = function()
                ischange = true
                local sep
                if type(args.texts[1]) == "table" then
                    args.texts = args.texts[1]
                    if args.encoding ~= "utf-8" then
                        sep = "\r\n"
                    else
                        sep = "\n"
                    end
                end
                
                args.text = table.concat(args.texts, sep)
                t[#t+1] = args.text
            end
            
            for c in f:lines() do
                ll = ll + 1
                if ll == args.line then
                    w()
                end
                if args.encoding ~= "utf-8" then
                    c = _ZM_.ConvEncoding(c, args.encoding, "utf-8")
                end
                c = c:gsub("[\r\n]", "")
                t[#t+1] = c
            end
            
            if args.line == ll + 1 then
                w()
            end
            
            if not ischange then
                f:close()
                return t
            end
            
            if args.encoding ~= "utf-8" then
                args.sep = "\r\n"
            else
                args.sep = "\n"
            end
                    
            args.text = table.concat(t, args.sep)
            
            if args.encoding ~= "utf-8" then
                args.text = _ZM_.ConvEncoding(args.text, "utf-8", args.encoding)
            end
            
            f:seek("set")
            f:write(args.text)
            f:close()
            return t
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.WriteLine()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end

--[=[
@fname:WriteLines
@cname:覆盖写入数组
@format:文件路径, 内容数组
@tname:覆盖写入 $2 到文件 $1 中
@note:覆盖写入文本内容数组到文件中, 文件编码为zmFile.SetEncoding()设置的编码
@ret:文本数组, table, 返回写入的文本数组, 失败返回null
@arg:文件路径, string, 要操作的文件路径
@arg:内容数组, table, 覆盖换行写入每一个元素
@exp:zmFile.Write("/sdcard/紫猫学院.txt", {"紫,", "猫", "学", "院"})
--]=]
function zimao.WriteLines(path, ...)
    local args, f = {path=path, texts={...}}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            
            if args.encoding == "auto" or args.encoding == "自动" then
                args.encoding = _ZM_.AutoEncoding(args.path)
            end
            
            f = io.open(args.path, "w")
            if f == nil then
                return nil
            end
            
            if type(args.texts[1]) == "table" then
                if args.encoding ~= "utf-8" then
                    args.sep = "\r\n"
                else
                    args.sep = "\n"
                end
                args.texts = args.texts[1]
            end
            
            args.text = table.concat(args.texts, args.sep)
            
            if args.encoding ~= "utf-8" then
                args.text = _ZM_.ConvEncoding(args.text, "utf-8", args.encoding)
            end
            
            f:write(args.text)
            f:close()
            return args.texts
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.WriteLines()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end

--[=[
@fname:WriteBinary
@cname:覆盖写入二进制
@format:文件路径, 二进制文本
@tname:覆盖写入二进制 $2 到文件 $1 中
@note:覆盖写入二进制到文件中
@ret:是否成功, boolean, 成功返回true, 失败或出错返回null
@arg:文件路径, string, 要操作的文件路径
@arg:二进制文本, string, 待写入的二进制内容
@exp:Dim bin = zmFile.ReadBinary("/sdcard/截图1.png")$n zmFile.WriteBinary "/sdcard/截图2.png", bin
--]=]
function zimao.WriteBinary(path, bin)
    local args, f = {path=path, text=bin}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            
            f = io.open(args.path, "wb")
            if f == nil then
                return nil
            end
            
            f:write(args.text)
            f:close()
            return true
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.WriteBinary()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end


--[=[
@fname:WriteAppend
@cname:追加写入文本
@format:文件路径, 内容[, ...]
@tname:追加 $2 到文件 $1 行中
@note:追加写入文本内容到文件中, 写完自动换行, 文件编码为zmFile.SetEncoding()设置的编码
@ret:是否成功, boolean, 成功返回true, 失败或出错返回null
@arg:文件路径, string, 要操作的文件路径
@arg:内容, string或table, 支持一个数组参数或者多个字符串参数$n 如果内容是一个数组参数, 则表示换行写入每个元素$n 如果内容是多个字符串参数, 则表示不换行依次写入每个字符串
@exp:zmFile.WriteAppend("/sdcard/紫猫学院.txt", "好好学习, 天天向上") //在末尾追加内容
--]=]
function zimao.WriteAppend(path, ...)
    local args, f = {path=path, texts={...}}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            

            if args.encoding == "auto" or args.encoding == "自动" then
                args.encoding = _ZM_.AutoEncoding(args.path)
            end
            
            f = io.open(args.path, "a")
            if f == nil then
                return nil
            end
            
            if type(args.texts[1]) == "table" then
                if args.encoding ~= "utf-8" then
                    args.sep = "\r\n"
                else
                    args.sep = "\n"
                end
                args.texts = args.texts[1]
            end
            
            args.text = table.concat(args.texts, args.sep)
            
            if args.encoding ~= "utf-8" then
                args.text = _ZM_.ConvEncoding(args.text, "utf-8", args.encoding)
            end
            
            f:write(args.text, args.encoding ~= "utf-8" and "\r\n" or "\n")
            f:close()
            return true
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.WriteAppend()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end

--[=[
@fname:DeleteLine
@cname:删除指定行
@format:文件路径, 指定行
@tname:删除文件 $1 的第 $2 行内容
@note:删除指定行文本内容, 文件编码为zmFile.SetEncoding()设置的编码
@ret:文本数组, table, 成功返回删除后的文本数组, 失败或出错返回null
@arg:文件路径, string, 要操作的文件路径
@arg:指定行, number, 表示删除第几行内容, 负数为倒数第几行内容
@exp:zmFile.DeleteLine("/sdcard/紫猫学院.txt", 1) //删除第一行内容
--]=]
function zimao.DeleteLine(path, line)
    local args, f = {path=path, line=line}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            
            if args.encoding == "auto" or args.encoding == "自动" then
                args.encoding = _ZM_.AutoEncoding(args.path)
            end
            
            f = io.open(args.path, "r")
            if f == nil then
                return nil
            end
            
            if args.line < 0 then
                local l = 0
                for _ in f:lines() do
                    l = l + 1
                end
                f:seek("set")
                args.line = l + args.line + 1
            end
            
            local ll, t, ischange = 0, {}
            for c in f:lines() do
                ll = ll + 1
                if ll ~= args.line then
                    if args.encoding ~= "utf-8" then
                        c = _ZM_.ConvEncoding(c, args.encoding, "utf-8")
                    end
                    c = c:gsub("[\r\n]", "")
                    t[#t+1] = c
                else
                    ischange = true
                end
            end
            f:close()
            if not ischange then
                return t
            end
            
            if args.encoding ~= "utf-8" then
                args.sep = "\r\n"
            else
                args.sep = "\n"
            end
                    
            args.text = table.concat(t, args.sep)
            
            if args.encoding ~= "utf-8" then
                args.text = _ZM_.ConvEncoding(args.text, "utf-8", args.encoding)
            end
            
            f = io.open(args.path, "w")
            if f == nil then
                return nil
            end
            f:write(args.text)
            f:close()
            return t
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.DeleteLine()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end

--[=[
@fname:ReplaceLine
@cname:替换指定行文本
@format:文件路径, 指定行, 内容[, ...]
@tname:替换文件 $1 的第 $2 行内容为 $3
@note:替换指定行文本, 文件编码为zmFile.SetEncoding()设置的编码
@ret:文本数组, table, 成功返回替换后的文本数组, 失败或出错返回null
@arg:文件路径, string, 要操作的文件路径
@arg:指定行, number, 表示替换第几行内容, 负数为倒数第几行内容
@arg:内容, string或table, 替换后的内容, 支持一个数组参数或者多个字符串参数$n 如果内容是一个数组参数, 则表示换行写入每个元素$n 如果内容是多个字符串参数, 则表示不换行依次写入每个字符串
@exp:zmFile.ReplaceLine("/sdcard/紫猫学院.txt", 2, "好好学习, 天天向上") //把第2行替换成指定内容
--]=]
function zimao.ReplaceLine(path, line, ...)
    local args, f = {path=path, line=line, texts={...}}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            
            if args.encoding == "auto" or args.encoding == "自动" then
                args.encoding = _ZM_.AutoEncoding(args.path)
            end
            
            f = io.open(args.path, "r")
            if f == nil then
                return nil
            end
            
            if args.line < 0 then
                local l = 0
                for _ in f:lines() do
                    l = l + 1
                end
                f:seek("set")
                args.line = l + args.line + 1
            end
            
            local ll, t, ischange = 0, {}
            for c in f:lines() do
                ll = ll + 1
                if ll ~= args.line then
                    if args.encoding ~= "utf-8" then
                        c = _ZM_.ConvEncoding(c, args.encoding, "utf-8")
                    end
                    c = c:gsub("[\r\n]", "")
                    t[#t+1] = c
                else
                    ischange = true
                    local sep
                    if type(args.texts[1]) == "table" then
                        args.texts = args.texts[1]
                        if args.encoding ~= "utf-8" then
                            sep = "\r\n"
                        else
                            sep = "\n"
                        end
                    end
                    
                    args.text = table.concat(args.texts, sep)
                    t[#t+1] = args.text
                end
            end
            f:close()
            if not ischange then
                return t
            end
            
            if args.encoding ~= "utf-8" then
                args.sep = "\r\n"
            else
                args.sep = "\n"
            end
                    
            args.text = table.concat(t, args.sep)
            
            if args.encoding ~= "utf-8" then
                args.text = _ZM_.ConvEncoding(args.text, "utf-8", args.encoding)
            end
            
            f = io.open(args.path, "w")
            if f == nil then
                return nil
            end
            f:write(args.text)
            f:close()
            return t
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.ReplaceLine()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end

--[=[
@fname:LinesNumber
@cname:获取文件总行数
@format:文件路径
@tname:获取文件 $1 的总行数
@note:获取文本总行数
@ret:总行数, number, 成功返回文件总行数, 失败返回null
@arg:文件路径, string, 要操作的文件路径
@exp:Dim line = zmFile.LinesNumber("/sdcard/紫猫学院.txt") //获取文件总行数
--]=]
function zimao.LinesNumber(path)
    local args, f = {path=path}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            
            f = io.open(args.path, "r")
            if f == nil then
                return nil
            end
            local line = 0
            for _ in f:lines() do
                line = line + 1
            end
            f:close()
            return line
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.LinesNumber()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end

--[=[
@fname:Bytes
@cname:获取文件字节大小
@format:文件路径
@tname:获取文件 $1 的总字节大小
@note:获取以字节为单位的文件大小
@ret:文件大小, number, 成功返回文件字节大小, 失败返回null
@arg:文件路径, string, 要操作的文件路径
@exp:Dim b = zmFile.Bytes("/sdcard/紫猫学院.txt") //获取文件字节大小
--]=]
function zimao.Bytes(path)
    local args, f = {path=path}
    return try {
        function()
            setmetatable(args, _ZM_.gFileAttr.mt)
            
            
            f = io.open(args.path, "r")
            if f == nil then
                return nil
            end
            local bytes = f:seek("end")
            f:close()
            return bytes
        end,
        catch {
            function (errors)
                traceprint("发生运行时错误！错误代码：zmFile.Bytes()，错误信息：", errors)
            end
        },
        finally {
            function ()
                if io.type(f) == "file" then f:close() end
            end
        }
    }
end


